package com.example.bdkipoc;

import android.animation.AnimatorSet;
import android.animation.ObjectAnimator;
import android.content.Context;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.animation.AccelerateDecelerateInterpolator;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Random;
import java.util.UUID;

public class QrisActivity extends AppCompatActivity {

    // Views
    private EditText editTextAmount;
    private Button initiatePaymentButton;
    private LinearLayout backNavigation;
    private TextView toolbarTitle;
    private TextView descriptionText;
    
    // Numpad buttons
    private TextView btn1, btn2, btn3, btn4, btn5, btn6, btn7, btn8, btn9, btn0, btn000;
    private TextView btnDelete; // Changed from ImageView to TextView
    
    // Only needed views for loading state
    private ProgressBar progressBar;
    private TextView statusTextView;
    private TextView referenceIdTextView;
    private LinearLayout initialPaymentLayout;

    // Data
    private StringBuilder currentAmount = new StringBuilder();
    private static final int MAX_AMOUNT_LENGTH = 12;
    private String transactionId;
    private String transactionUuid;
    private String referenceId;
    private int amount;
    private JSONObject midtransResponse;
    
    // Animation
    private Handler animationHandler = new Handler(Looper.getMainLooper());

    private static final String BACKEND_BASE = "https://be-edc.msvc.app";
    private static final String MIDTRANS_CHARGE_URL = "https://api.sandbox.midtrans.com/v2/charge";
    private static final String MIDTRANS_AUTH = "Basic U0ItTWlkLXNlcnZlci1JM2RJWXdIRzVuamVMeHJCMVZ5endWMUM="; // Replace with your actual key
    private static final String WEBHOOK_URL = "https://be-edc.msvc.app/webhooks/midtrans";

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        // Set status bar color programmatically
        setStatusBarColor();
        
        setContentView(R.layout.activity_qris);

        initializeViews();
        setupClickListeners();
        setupInitialStates();
        
        // Generate reference ID
        referenceId = "ref-" + generateRandomString(8);
        referenceIdTextView.setText(referenceId);

        // Initially hide the progress and status views
        progressBar.setVisibility(View.GONE);
        statusTextView.setVisibility(View.GONE);
        initialPaymentLayout.setVisibility(View.VISIBLE);
    }

    private void setStatusBarColor() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            Window window = getWindow();
            window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
            window.setStatusBarColor(Color.parseColor("#E31937")); // Red color
            
            // Make status bar icons white (for dark red background)
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                View decorView = window.getDecorView();
                decorView.setSystemUiVisibility(0); // Clear light status bar flag
            }
        }
    }

    private void initializeViews() {
        // New UI components (similar to PaymentActivity)
        editTextAmount = findViewById(R.id.editTextAmount);
        initiatePaymentButton = findViewById(R.id.initiatePaymentButton);
        backNavigation = findViewById(R.id.back_navigation);
        toolbarTitle = findViewById(R.id.toolbarTitle);
        descriptionText = findViewById(R.id.descriptionText);

        // Numpad buttons
        btn1 = findViewById(R.id.btn1);
        btn2 = findViewById(R.id.btn2);
        btn3 = findViewById(R.id.btn3);
        btn4 = findViewById(R.id.btn4);
        btn5 = findViewById(R.id.btn5);
        btn6 = findViewById(R.id.btn6);
        btn7 = findViewById(R.id.btn7);
        btn8 = findViewById(R.id.btn8);
        btn9 = findViewById(R.id.btn9);
        btn0 = findViewById(R.id.btn0);
        btn000 = findViewById(R.id.btn000);
        btnDelete = findViewById(R.id.btnDelete); // Now TextView
        
        // Only needed views for loading state
        progressBar = findViewById(R.id.progressBar);
        statusTextView = findViewById(R.id.statusTextView);
        referenceIdTextView = findViewById(R.id.referenceIdTextView);
        
        // Main content layout (replaces initialPaymentLayout)
        initialPaymentLayout = findViewById(R.id.mainContentLayout);
    }

    private void setupClickListeners() {
        // Back navigation
        backNavigation.setOnClickListener(v -> {
            addClickAnimation(v);
            navigateBack();
        });

        toolbarTitle.setOnClickListener(v -> {
            addClickAnimation(v);
            navigateBack();
        });

        // Numpad listeners
        btn1.setOnClickListener(v -> handleNumpadClick(v, "1"));
        btn2.setOnClickListener(v -> handleNumpadClick(v, "2"));
        btn3.setOnClickListener(v -> handleNumpadClick(v, "3"));
        btn4.setOnClickListener(v -> handleNumpadClick(v, "4"));
        btn5.setOnClickListener(v -> handleNumpadClick(v, "5"));
        btn6.setOnClickListener(v -> handleNumpadClick(v, "6"));
        btn7.setOnClickListener(v -> handleNumpadClick(v, "7"));
        btn8.setOnClickListener(v -> handleNumpadClick(v, "8"));
        btn9.setOnClickListener(v -> handleNumpadClick(v, "9"));
        btn0.setOnClickListener(v -> handleNumpadClick(v, "0"));
        btn000.setOnClickListener(v -> handleNumpadClick(v, "000"));

        // Delete button
        btnDelete.setOnClickListener(v -> {
            addClickAnimation(v);
            deleteLastDigit();
        });

        // Original QRIS buttons - only initiate payment needed
        initiatePaymentButton.setOnClickListener(v -> {
            if (initiatePaymentButton.isEnabled()) {
                addButtonClickAnimation(v);
                createTransaction();
            }
        });
    }

    private void setupInitialStates() {
        // Initially hide amount input and show description
        editTextAmount.setVisibility(View.GONE);
        descriptionText.setVisibility(View.VISIBLE);
        
        // Set initial button state
        updateButtonState();
        
        // Disable EditText input (only numpad input allowed)
        editTextAmount.setFocusable(false);
        editTextAmount.setClickable(false);
        editTextAmount.setCursorVisible(false);
    }

    private void navigateBack() {
        finish();
    }

    private void handleNumpadClick(View view, String digit) {
        addClickAnimation(view);
        addDigit(digit);
    }

    private void addDigit(String digit) {
        // Validate input length
        if (currentAmount.length() >= MAX_AMOUNT_LENGTH) {
            showToast("Maksimal " + MAX_AMOUNT_LENGTH + " digit");
            return;
        }

        // Handle leading zeros
        if (currentAmount.length() == 0) {
            if (digit.equals("000")) {
                // Don't allow 000 as first input
                return;
            }
            currentAmount.append(digit);
        } else if (currentAmount.length() == 1 && currentAmount.toString().equals("0")) {
            if (!digit.equals("000")) {
                // Replace single 0 with new digit
                currentAmount = new StringBuilder(digit);
            } else {
                return;
            }
        } else {
            currentAmount.append(digit);
        }

        updateAmountDisplay();
        updateButtonState();
        addInputFeedback();
    }

    private void deleteLastDigit() {
        if (currentAmount.length() > 0) {
            String current = currentAmount.toString();
            
            // If current ends with 000, remove all three digits
            if (current.endsWith("000") && current.length() >= 3) {
                currentAmount.delete(currentAmount.length() - 3, currentAmount.length());
            } else {
                currentAmount.deleteCharAt(currentAmount.length() - 1);
            }
            
            updateAmountDisplay();
            updateButtonState();
            addDeleteFeedback();
        }
    }

    private void updateAmountDisplay() {
        String amount = currentAmount.toString();
        
        if (amount.isEmpty() || amount.equals("0")) {
            // Show description text, hide amount input
            editTextAmount.setVisibility(View.GONE);
            descriptionText.setVisibility(View.VISIBLE);
        } else {
            // Show amount input, hide description text
            String formattedAmount = formatCurrency(amount);
            editTextAmount.setText(formattedAmount);
            editTextAmount.setVisibility(View.VISIBLE);
            descriptionText.setVisibility(View.GONE);
        }
    }

    private String formatCurrency(String amount) {
        if (TextUtils.isEmpty(amount) || amount.equals("0")) {
            return "";
        }
        
        try {
            long number = Long.parseLong(amount);
            return String.format("%,d", number).replace(',', '.');
        } catch (NumberFormatException e) {
            return amount;
        }
    }

    private void updateButtonState() {
        boolean hasValidAmount = currentAmount.length() > 0 && 
                                !currentAmount.toString().equals("0") &&
                                !currentAmount.toString().isEmpty();
        
        initiatePaymentButton.setEnabled(hasValidAmount);
        
        // Use MaterialButton's backgroundTint property
        com.google.android.material.button.MaterialButton materialButton = 
            (com.google.android.material.button.MaterialButton) initiatePaymentButton;
        
        if (hasValidAmount) {
            // Active state - red background like in the XML
            materialButton.setBackgroundTintList(android.content.res.ColorStateList.valueOf(Color.parseColor("#DE0701")));
            materialButton.setTextColor(Color.WHITE);
            materialButton.setAlpha(1.0f);
        } else {
            // Inactive state - gray background
            materialButton.setBackgroundTintList(android.content.res.ColorStateList.valueOf(Color.parseColor("#E8E8E8")));
            materialButton.setTextColor(Color.parseColor("#999999"));
            materialButton.setAlpha(0.6f);
        }
    }

    private void createTransaction() {
        String amountText = currentAmount.toString();
        
        if (TextUtils.isEmpty(amountText) || amountText.equals("0")) {
            showToast("Masukkan jumlah pembayaran");
            return;
        }

        try {
            long amountValue = Long.parseLong(amountText);
            
            // Validate minimum amount
            if (amountValue < 1000) {
                showToast("Minimal pembayaran Rp 1.000");
                return;
            }

            // Validate maximum amount
            if (amountValue > 999999999L) {
                showToast("Maksimal pembayaran Rp 999.999.999");
                return;
            }

            // Set amount for transaction
            amount = (int) amountValue;
            
            // Show loading state
            progressBar.setVisibility(View.VISIBLE);
            statusTextView.setText("Creating transaction...");
            statusTextView.setVisibility(View.VISIBLE);
            initiatePaymentButton.setEnabled(false);
            
            new CreateTransactionTask().execute();
            
        } catch (NumberFormatException e) {
            showToast("Format jumlah tidak valid");
        }
    }

    private void displayQrCode(String qrImageUrl) {
        // This method is no longer needed since we navigate to QrisResultActivity
        // Keeping it for compatibility but it won't be called
    }

    private void simulateWebhook() {
        // This method is no longer needed since we navigate to QrisResultActivity
        // QrisResultActivity handles webhook simulation
    }

    private void showSuccessScreen() {
        // This method is no longer needed since we navigate to QrisResultActivity
        // QrisResultActivity handles success display
    }

    private String generateRandomString(int length) {
        String chars = "abcdefghijklmnopqrstuvwxyz0123456789";
        StringBuilder sb = new StringBuilder();
        Random random = new Random();
        for (int i = 0; i < length; i++) {
            int index = random.nextInt(chars.length());
            sb.append(chars.charAt(index));
        }
        return sb.toString();
    }

    private String getServerKey() {
        // MIDTRANS_AUTH = 'Basic base64string'
        String base64 = MIDTRANS_AUTH.replace("Basic ", "");
        String decoded = android.util.Base64.decode(base64, android.util.Base64.DEFAULT).toString();
        // Format is usually 'SB-Mid-server-xxxx:'. Remove trailing colon if present.
        return decoded.replace(":\n", "");
    }

    private String generateSignature(String orderId, String statusCode, String grossAmount, String serverKey) {
        String input = orderId + statusCode + grossAmount + serverKey;
        try {
            java.security.MessageDigest md = java.security.MessageDigest.getInstance("SHA-512");
            byte[] messageDigest = md.digest(input.getBytes());
            StringBuilder hexString = new StringBuilder();
            for (byte b : messageDigest) {
                String hex = Integer.toHexString(0xff & b);
                if (hex.length() == 1) hexString.append('0');
                hexString.append(hex);
            }
            return hexString.toString();
        } catch (java.security.NoSuchAlgorithmException e) {
            return "";
        }
    }

    // Animation methods
    private void addClickAnimation(View view) {
        ObjectAnimator scaleX = ObjectAnimator.ofFloat(view, "scaleX", 1f, 0.95f, 1f);
        ObjectAnimator scaleY = ObjectAnimator.ofFloat(view, "scaleY", 1f, 0.95f, 1f);
        
        AnimatorSet animatorSet = new AnimatorSet();
        animatorSet.playTogether(scaleX, scaleY);
        animatorSet.setDuration(150);
        animatorSet.start();
    }

    private void addButtonClickAnimation(View view) {
        ObjectAnimator scaleX = ObjectAnimator.ofFloat(view, "scaleX", 1f, 0.98f, 1f);
        ObjectAnimator scaleY = ObjectAnimator.ofFloat(view, "scaleY", 1f, 0.98f, 1f);
        
        AnimatorSet animatorSet = new AnimatorSet();
        animatorSet.playTogether(scaleX, scaleY);
        animatorSet.setDuration(200);
        animatorSet.start();
    }

    private void addInputFeedback() {
        ObjectAnimator fadeIn = ObjectAnimator.ofFloat(editTextAmount, "alpha", 0.7f, 1f);
        fadeIn.setDuration(200);
        fadeIn.start();
    }

    private void addDeleteFeedback() {
        ObjectAnimator shake = ObjectAnimator.ofFloat(editTextAmount, "translationX", 0f, -10f, 10f, 0f);
        shake.setDuration(300);
        shake.start();
    }

    // Utility methods
    private void showToast(String message) {
        Toast.makeText(this, message, Toast.LENGTH_SHORT).show();
    }

    @Override
    public void onBackPressed() {
        // Simple back navigation
        navigateBack();
    }
    
    private class CreateTransactionTask extends AsyncTask<Void, Void, Boolean> {
        private String errorMessage;
        
        @Override
        protected Boolean doInBackground(Void... voids) {
            try {
                // Generate a UUID for the transaction
                transactionUuid = UUID.randomUUID().toString();
                
                // Create transaction JSON payload
                JSONObject payload = new JSONObject();
                payload.put("type", "PAYMENT");
                payload.put("channel_category", "RETAIL_OUTLET");
                payload.put("channel_code", "QRIS");
                payload.put("reference_id", referenceId);
                
                Log.d("MidtransCharge", "Amount for transaction: " + amount);
                
                payload.put("amount", amount);
                payload.put("cashflow", "MONEY_IN");
                payload.put("status", "INIT");
                payload.put("device_id", 1);
                payload.put("transaction_uuid", transactionUuid);
                payload.put("transaction_time_seconds", 0.0);
                payload.put("device_code", "PB4K252T00021");
                payload.put("merchant_name", "Marcel Panjaitan");
                payload.put("mid", "71000026521");
                payload.put("tid", "73001500");
                
                // Make the API call
                URL url = new URI(BACKEND_BASE + "/transactions").toURL();
                HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                conn.setRequestMethod("POST");
                conn.setRequestProperty("Content-Type", "application/json");
                conn.setRequestProperty("Accept", "application/json");
                conn.setDoOutput(true);
                
                try (OutputStream os = conn.getOutputStream()) {
                    byte[] input = payload.toString().getBytes("utf-8");
                    os.write(input, 0, input.length);
                }
                
                int responseCode = conn.getResponseCode();
                if (responseCode == 200 || responseCode == 201) {
                    // Read the response
                    BufferedReader br = new BufferedReader(new InputStreamReader(conn.getInputStream(), "utf-8"));
                    StringBuilder response = new StringBuilder();
                    String responseLine;
                    while ((responseLine = br.readLine()) != null) {
                        response.append(responseLine.trim());
                    }
                    
                    // Parse the response to get transaction ID
                    JSONObject jsonResponse = new JSONObject(response.toString());
                    JSONObject data = jsonResponse.getJSONObject("data");
                    transactionId = String.valueOf(data.getInt("id"));
                    
                    // Now generate QRIS via Midtrans
                    return generateQris(amount);
                } else {
                    // Read error response
                    BufferedReader br = new BufferedReader(new InputStreamReader(conn.getErrorStream(), "utf-8"));
                    StringBuilder response = new StringBuilder();
                    String responseLine;
                    while ((responseLine = br.readLine()) != null) {
                        response.append(responseLine.trim());
                    }
                    errorMessage = "Error creating transaction: " + response.toString();
                    return false;
                }
            } catch (Exception e) {
                Log.e("MidtransCharge", "Exception: " + e.getMessage(), e);
                errorMessage = "Unexpected error: " + e.getMessage();
                return false;
            }
        }
        
        private boolean generateQris(int amount) {
            try {
                // Create QRIS charge JSON payload
                JSONObject payload = new JSONObject();
                payload.put("payment_type", "qris");
                
                JSONObject transactionDetails = new JSONObject();
                transactionDetails.put("order_id", transactionUuid);
                transactionDetails.put("gross_amount", amount);
                payload.put("transaction_details", transactionDetails);
                
                // Log the request details
                Log.d("MidtransCharge", "URL: " + MIDTRANS_CHARGE_URL);
                Log.d("MidtransCharge", "Authorization: " + MIDTRANS_AUTH);
                Log.d("MidtransCharge", "Accept: application/json");
                Log.d("MidtransCharge", "Content-Type: application/json");
                Log.d("MidtransCharge", "X-Override-Notification: " + WEBHOOK_URL);
                Log.d("MidtransCharge", "Payload: " + payload.toString());

                // Make the API call to Midtrans
                URL url = new URI(MIDTRANS_CHARGE_URL).toURL();
                HttpURLConnection conn = (HttpURLConnection) url.openConnection();
                conn.setRequestMethod("POST");
                conn.setRequestProperty("Accept", "application/json");
                conn.setRequestProperty("Content-Type", "application/json");
                conn.setRequestProperty("Authorization", MIDTRANS_AUTH); 
                conn.setRequestProperty("X-Override-Notification", WEBHOOK_URL);
                conn.setDoOutput(true);
                
                try (OutputStream os = conn.getOutputStream()) {
                    byte[] input = payload.toString().getBytes("utf-8");
                    os.write(input, 0, input.length);
                }
                
                int responseCode = conn.getResponseCode();
                if (responseCode == 200 || responseCode == 201) {
                    InputStream inputStream = conn.getInputStream();
                    if (inputStream != null) {
                        BufferedReader br = new BufferedReader(new InputStreamReader(inputStream, "utf-8"));
                        StringBuilder response = new StringBuilder();
                        String responseLine;
                        while ((responseLine = br.readLine()) != null) {
                            response.append(responseLine.trim());
                        }
                        
                        // Parse the response
                        midtransResponse = new JSONObject(response.toString());
                        return true;
                    } else {
                        Log.e("MidtransCharge", "HTTP " + responseCode + ": No input stream available");
                        errorMessage = "Error generating QRIS: HTTP " + responseCode + ": No input stream available";
                        return false;
                    }
                } else {
                    InputStream errorStream = conn.getErrorStream();
                    if (errorStream != null) {
                        BufferedReader br = new BufferedReader(new InputStreamReader(errorStream, "utf-8"));
                        StringBuilder errorResponse = new StringBuilder();
                        String responseLine;
                        while ((responseLine = br.readLine()) != null) {
                            errorResponse.append(responseLine.trim());
                        }
                        Log.e("MidtransCharge", "HTTP " + responseCode + ": " + errorResponse.toString());
                        errorMessage = "Error generating QRIS: HTTP " + responseCode + ": " + errorResponse.toString();
                    } else {
                        Log.e("MidtransCharge", "HTTP " + responseCode + ": No error stream available");
                        errorMessage = "Error generating QRIS: HTTP " + responseCode + ": No error stream available";
                    }
                    return false;
                }
            } catch (Exception e) {
                Log.e("MidtransCharge", "Exception: " + e.getMessage(), e);
                errorMessage = "Unexpected error: " + e.getMessage();
                return false;
            }
        }
        
        @Override
        protected void onPostExecute(Boolean success) {
            if (success && midtransResponse != null) {
                try {
                    // Extract needed values from midtransResponse
                    JSONObject actions = midtransResponse.getJSONArray("actions").getJSONObject(0);
                    String qrImageUrl = actions.getString("url");
                    
                    // Extract transaction_id
                    String transactionId = midtransResponse.getString("transaction_id");
                    String transactionTime = midtransResponse.getString("transaction_time");
                    String acquirer = midtransResponse.getString("acquirer");
                    String merchantId = midtransResponse.getString("merchant_id");
                    String exactGrossAmount = midtransResponse.getString("gross_amount");
                    
                    // Log everything before launching activity
                    Log.d("MidtransCharge", "Creating QrisResultActivity intent with:");
                    Log.d("MidtransCharge", "qrImageUrl: " + qrImageUrl);
                    Log.d("MidtransCharge", "amount: " + amount);
                    Log.d("MidtransCharge", "referenceId: " + referenceId);
                    Log.d("MidtransCharge", "transactionUuid (orderId): " + transactionUuid);
                    Log.d("MidtransCharge", "transaction_id: " + transactionId);
                    Log.d("MidtransCharge", "exactGrossAmount: " + exactGrossAmount);
                    
                    // Launch QrisResultActivity instead of showing QR inline
                    Intent intent = new Intent(QrisActivity.this, QrisResultActivity.class);
                    intent.putExtra("qrImageUrl", qrImageUrl);
                    intent.putExtra("amount", amount);
                    intent.putExtra("referenceId", referenceId);
                    intent.putExtra("orderId", transactionUuid); // Order ID
                    intent.putExtra("transactionId", transactionId); // Actual Midtrans transaction_id
                    intent.putExtra("grossAmount", exactGrossAmount); // Exact gross amount from response
                    intent.putExtra("transactionTime", transactionTime); // For timestamp
                    intent.putExtra("acquirer", acquirer);
                    intent.putExtra("merchantId", merchantId);
                    
                    try {
                        startActivity(intent);
                        // Finish this activity so user can't go back to input form
                        finish();
                    } catch (Exception e) {
                        Log.e("MidtransCharge", "Failed to start QrisResultActivity: " + e.getMessage(), e);
                        Toast.makeText(QrisActivity.this, "Error launching QR display: " + e.getMessage(), Toast.LENGTH_LONG).show();
                        resetToInitialState();
                    }
                    
                } catch (JSONException e) {
                    Log.e("MidtransCharge", "QRIS response JSON error: " + e.getMessage(), e);
                    Toast.makeText(QrisActivity.this, "Error processing QRIS response", Toast.LENGTH_LONG).show();
                    resetToInitialState();
                }
            } else {
                String message = (errorMessage != null && !errorMessage.isEmpty()) ? errorMessage : "Unknown error occurred. Please check Logcat for details.";
                Toast.makeText(QrisActivity.this, message, Toast.LENGTH_LONG).show();
                resetToInitialState();
            }
            progressBar.setVisibility(View.GONE);
        }
    }
    
    private void resetToInitialState() {
        // Reset to show the input form again
        progressBar.setVisibility(View.GONE);
        statusTextView.setVisibility(View.GONE);
        initiatePaymentButton.setEnabled(true);
        updateButtonState();
    }
    
    // Remove DownloadImageTask - no longer needed
    
    // Remove SimulateWebhookTask - no longer needed

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (animationHandler != null) {
            animationHandler.removeCallbacksAndMessages(null);
        }
    }

    // Public methods for testing
    public String getCurrentAmount() {
        return currentAmount.toString();
    }

    public boolean isInitiateButtonEnabled() {
        return initiatePaymentButton.isEnabled();
    }
}