package com.example.bdkipoc;

import android.content.Intent;
import android.graphics.Color;
import android.os.Build;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;
import androidx.appcompat.app.AppCompatActivity;

public class ReceiptActivity extends AppCompatActivity {

    // Views
    private LinearLayout backNavigation;
    private ImageView backArrow;
    private TextView toolbarTitle;
    
    // Receipt details
    private TextView merchantName;
    private TextView merchantLocation;
    private TextView midText;
    private TextView tidText;
    private TextView transactionNumber;
    private TextView transactionDate;
    private TextView paymentMethod;
    private TextView cardType;
    private TextView transactionTotal;
    private TextView taxPercentage;
    private TextView serviceFee;
    private TextView finalTotal;
    
    // Action buttons
    private LinearLayout printButton;
    private LinearLayout emailButton;
    private Button finishButton;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        
        // Set status bar color
        setStatusBarColor();
        
        setContentView(R.layout.activity_receipt);
        
        initializeViews();
        setupClickListeners();
        loadTransactionData();
    }

    private void setStatusBarColor() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            Window window = getWindow();
            window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
            window.setStatusBarColor(Color.parseColor("#E31937")); // Red color
            
            // Make status bar icons white (for dark red background)
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                View decorView = window.getDecorView();
                decorView.setSystemUiVisibility(0); // Clear light status bar flag
            }
        }
    }

    private void initializeViews() {
        // Navigation
        backNavigation = findViewById(R.id.back_navigation);
        backArrow = findViewById(R.id.backArrow);
        toolbarTitle = findViewById(R.id.toolbarTitle);
        
        // Receipt details
        merchantName = findViewById(R.id.merchant_name);
        merchantLocation = findViewById(R.id.merchant_location);
        midText = findViewById(R.id.mid_text);
        tidText = findViewById(R.id.tid_text);
        transactionNumber = findViewById(R.id.transaction_number);
        transactionDate = findViewById(R.id.transaction_date);
        paymentMethod = findViewById(R.id.payment_method);
        cardType = findViewById(R.id.card_type);
        transactionTotal = findViewById(R.id.transaction_total);
        taxPercentage = findViewById(R.id.tax_percentage);
        serviceFee = findViewById(R.id.service_fee);
        finalTotal = findViewById(R.id.final_total);
        
        // Action buttons - Updated to LinearLayout
        printButton = findViewById(R.id.print_button);
        emailButton = findViewById(R.id.email_button);
        finishButton = findViewById(R.id.finish_button);
    }

    private void setupClickListeners() {
        // Back navigation - Goes back to previous activity (PaymentActivity)
        backNavigation.setOnClickListener(v -> navigateBackToPrevious());
        backArrow.setOnClickListener(v -> navigateBackToPrevious());
        toolbarTitle.setOnClickListener(v -> navigateBackToPrevious());
        
        // Action buttons
        printButton.setOnClickListener(v -> handlePrintReceipt());
        emailButton.setOnClickListener(v -> handleEmailReceipt());
        finishButton.setOnClickListener(v -> handleFinish());
    }

    private void loadTransactionData() {
        Intent intent = getIntent();
        if (intent != null) {
            // Load transaction data from intent
            String amount = intent.getStringExtra("transaction_amount");
            String merchantNameStr = intent.getStringExtra("merchant_name");
            String merchantLocationStr = intent.getStringExtra("merchant_location");
            String transactionId = intent.getStringExtra("transaction_id");
            String transactionDateStr = intent.getStringExtra("transaction_date");
            String paymentMethodStr = intent.getStringExtra("payment_method");
            String cardTypeStr = intent.getStringExtra("card_type");
            String taxPercentageStr = intent.getStringExtra("tax_percentage");
            String serviceFeeStr = intent.getStringExtra("service_fee");
            
            // Set merchant info
            if (!TextUtils.isEmpty(merchantNameStr)) {
                merchantName.setText(merchantNameStr);
            }
            if (!TextUtils.isEmpty(merchantLocationStr)) {
                merchantLocation.setText(merchantLocationStr);
            }
            
            // Set transaction details
            if (!TextUtils.isEmpty(transactionId)) {
                transactionNumber.setText(transactionId);
            }
            if (!TextUtils.isEmpty(transactionDateStr)) {
                transactionDate.setText(transactionDateStr);
            }
            if (!TextUtils.isEmpty(paymentMethodStr)) {
                paymentMethod.setText(paymentMethodStr);
            }
            if (!TextUtils.isEmpty(cardTypeStr)) {
                cardType.setText(cardTypeStr);
            }
            if (!TextUtils.isEmpty(taxPercentageStr)) {
                taxPercentage.setText(taxPercentageStr);
            }
            if (!TextUtils.isEmpty(serviceFeeStr)) {
                serviceFee.setText(serviceFeeStr);
            }
            
            // Calculate and set amounts
            if (!TextUtils.isEmpty(amount)) {
                try {
                    long amountValue = Long.parseLong(amount);
                    String formattedAmount = formatCurrency(amountValue);
                    
                    transactionTotal.setText(formattedAmount);
                    
                    // Calculate total with tax and service fee
                    long serviceFeeLong = TextUtils.isEmpty(serviceFeeStr) ? 0 : Long.parseLong(serviceFeeStr);
                    long totalWithFee = amountValue + serviceFeeLong;
                    
                    finalTotal.setText(formatCurrency(totalWithFee));
                    
                } catch (NumberFormatException e) {
                    transactionTotal.setText(amount);
                    finalTotal.setText(amount);
                }
            }
            
            // Set MID and TID (you can customize these)
            midText.setText("1234567890");
            tidText.setText("1234567890");
        }
    }

    private String formatCurrency(long amount) {
        return String.format("%,d", amount).replace(',', '.');
    }

    private void handlePrintReceipt() {
        // Handle print receipt action
        // In real app, this would integrate with printer
        showToast("Mencetak struk...");
    }

    private void handleEmailReceipt() {
        // Handle email receipt action
        // In real app, this would open email intent
        showToast("Mengirim email...");
    }

    private void handleFinish() {
        // Navigate to MainActivity/Home Page when "Selesai" button is pressed
        navigateToHomePage();
    }

    private void navigateBackToPrevious() {
        // Navigate back to PaymentActivity (previous activity)
        Intent intent = new Intent(this, PaymentActivity.class);
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_NEW_TASK);
        startActivity(intent);
        finish();
    }

    private void navigateToHomePage() {
        // Navigate to MainActivity/Home Page when "Selesai" button is pressed
        Intent intent = new Intent(this, MainActivity.class);
        
        // Clear all previous activities from the stack and start fresh
        intent.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_NEW_TASK);
        
        // Optional: Add success message to show in MainActivity
        intent.putExtra("transaction_completed", true);
        intent.putExtra("transaction_amount", getIntent().getStringExtra("transaction_amount"));
        
        startActivity(intent);
        finish();
        
        // Show success message
        showToast("Transaksi berhasil diselesaikan!");
    }

    private void showToast(String message) {
        android.widget.Toast.makeText(this, message, android.widget.Toast.LENGTH_SHORT).show();
    }

    @Override
    public void onBackPressed() {
        // Back button behavior - goes back to previous activity
        navigateBackToPrevious();
    }
}